# linksubmit.pl - Submit Link Job
#
# Authors:  Jane Huang
# Last updated:  7/14/07
#                Xiao Lin Zhang, 7/06/09
#      Xiao Lin Zhang 28/6/10
#
# (c) Copyright IBM Corp. 2008
#
# The common routines used in all the build scripts are contained in the 
# bccpbmod.pm module.
#
# This script is spawned from the bccpmbld_main.pl script for each COBOL
# file that was selected to be linked.  
#
# The calling syntax for this script is:
# 	Usage: linksubmit.pl [-1] -d directory_path -f filename
# Where;
# 	-1 indicates that the linked-edit list is one file which causes
# 	   the log output to be displayed to stdout as well as the log file
#   -i identifies the directory where the profileUSER are located
# 	-d identifies the directory where the element to be compiled exists
# 	-f identifies the file to be compiled
#
#
#
# You must modify this script before attempting to use parameterized build.
# Sections that require modification and the instructions for modifying them
# are flagged with the following comment lines:
#
# ******************************************************************************
# * Customization instructions:                                                *
# * [instructions here]                                                        *
# ****************************************************************************** 
use File::Basename;
use File::Copy;
use POSIX qw(strftime);
use Getopt::Std;
use bccpbmod;

my $debug      = $bccpbmod::debug;     # for verbose debugging output
my $debug      = 1;     # for verbose debugging output

my $rccVerbosity = "";
$rccVerbosity = '-V -V -V' if $debug;

my $sysout       = $bccpbmod::sysout;
$sysout = 'RCCEXT=RCCOUT,DISP=(NEW,CATLG,DELETE),' if $debug;
my $sysout1 = 'SPACE=(32000,(30,30))' if $debug;


my $jcl;	    # JCL filename
my $ncpycnt = 7;    # number of CICS NewCopy steps
my $lcnt    = 0;    # general loop counter

# #############################################################################
# process input arguments
#
#
  
my $usage = "linksubmit [-1] -i profileUSER_Path -d dir -f file";         

my $compile_1 = 0;  # Link-edit from main consisted of one file
my $dir_file  = ''; # directory where the file exists
my $filename  = ''; # filename - full or relative path

if (!getopts('1i:d:f:') || $opt_h) {
    print "$usage \n";
    exit(0);
}


# *******************************************************************************
# * Required customization:                                                     *
# * Judge OS type to determine the delimiter embedded in path, you can modify it*
# * according to you site's requirement or you can hardcode delimiter here.     *
# ******************************************************************************* 
my $delimiter = '';
my $pcid = '';
my $current_dir = '';

$pcid=$ENV{USERNAME} || $ENV{USER} || $ENV{LOGNAME};

my $OS = $^O;
if ($OS =~/.*Win.*/){	#For Windows operating system
	$delimiter = "\\";
	$current_dir = `cd`;
}elsif ($OS =~/.*n.*x/ or $OS eq "hpux" or $OS eq "solaris"){	# For Unix & Linux operating system
	$delimiter = "/";
	$current_dir = `pwd`;
}elsif ($OS eq "aix"){	#For AIX operating system
	$delimiter = "/";
	$current_dir = `pwd`;
}
chomp $current_dir;

$compile_1 = $opt_1;
$dir_file  = $opt_d;
$filename  = $opt_f;
$Path_profileUSER = $opt_i;

print "current dir =  $current_dir \n" if $debug;
print "Link script: One file = $compile_1 \n" if $debug;
print "Link script: filename = $filename \n" if $debug;
print "Link script: dir_file = $dir_file \n" if $debug;


($scriptdir,$lifecycle,$EDfile,$VIEWfile,$listdir,$logdir,$perlexe) = bccpbmod::getprofileUSER($Path_profileUSER,$delimiter);

# #############################################################################
# Get the environment variables from the common module
# These variables establish the distributed and mainframe environment
# and must be modified for each installation.  The values for these 
# variables are held in projectED file.
# These varibles are used to point to the Z/OS infrastructure/environment/mapped 
# Life cycle that the project will take. This includes SYSLIB concatenation to be
# used for each environment ie: TEST, QA, PROD. What target loadlibs are to be used
# DB2 subsystem, CICS region, IMS region etc. They are declared at the project level
# the are queried and used here.
# *******************************************************************************
# * Optional customization:                                                     *
# * You can use the optional variables or add new variables based on your site's*
# * needs.                                                                      *
# *******************************************************************************
my ($env, $db2sys, $os390_hostname, $os390_port, $stage,
$copylib1, $copylib2, $copylib3, $copylib4,
$syslib1, $syslib2, $syslib3,
#$syslib4, $steplib1, $steplib2,			 # optional
#$hostpds_prefix, $hostpds_affix, $cbllib,	         # optional 
$loadlib, $c390llib, $dbrmlib, $tgtlib, $db2_owner) = bccpbmod::setEnvironmentVar($EDfile,$lifecycle,$delimiter);

print "Env: $env \n" if $debug;

# #############################################################################
# Translate the distributed username to a mainframe ID
#

my $mfid = bccpbmod::getUserID($scriptdir,$pcid,$delimiter);  
bccpbmod::my_die('Unknown User.') if ($mfid eq "");


# *******************************************************************************
# * Optional customization:                                                     *
# * The following code can be used to generate the PDS if the type attribute    *
# * ZCCENV is set with the $hostpds_prefix and $hostpds_affix.                  *
# *******************************************************************************

  my $name = $mfid;
#  my $affix ="";
#  my $prefix ="";
#  if (($hostpds_prefix ne '') || ($hostpds_affix ne '')) {
#     if ($hostpds_prefix ne '') {
#        $prefix = $hostpds_prefix . ".";
#     }
#     if ($hostpds_affix ne '') {
#        $affix = "." . $hostpds_affix ;
#     }
#     $name = $prefix . $mfid . $affix ;
#   } 
################################################################################
print "PCID: $pcid, MFID: $mfid, NAME: $name\n" if $debug;


# *******************************************************************************
# * Required customization:                                                     *
# * define location for jcl which will be generated during build request        *
# ******************************************************************************* 
my $jcldir = $dir_file . $delimiter . $mfid; #full name path for jcl location
# #############################################################################
# Cleanup JCL files from the previous execution
#
#bccpbmod::cleanup($jcldir,$delimiter);

# #############################################################################
# determine the member name from input filename. The member name is
# just the filename without the front path and extension.
#
my $mem_ext = basename($filename);
my $dot      = rindex( $mem_ext, '.' );
my $mem      = substr( $mem_ext, 0, $dot );
my $ext      = substr( $mem_ext, $dot+1);

my $jcldir = $dir_file.$delimiter.$mfid.$delimiter.$mem;
printf( "processing file (%s)\n", $filename ) if $debug;
printf( "  mem (%s) ext (%s)\n", $mem, $ext ) if $debug;


# #############################################################################
# Retrieve file information from element attribute file projectVIEW
#
my %attrHash = bccpbmod::getAttrs($VIEWfile,$filename);

while (($attr, $value) = each %attrHash) {
	print "Attribute: $attr\tValue: $value\n";
	if ($compile_1) {
		print STDERR "Attribute: $attr\tValue: $value\n";
	}
}

# *******************************************************************************
# * Required customization:                                                     *
# * The following code has to be modified to meet your site's requirement.      *
# *******************************************************************************
# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< main >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
#
# The main routine continues by calling each of the specific job routines.
# If any of the routines fail, this scripts exits with an error message.
                     
 bccpbmod::my_die('Link Error.')                      if ( LinkProg() > 4 ); 

exit ;

##############################################################
# The following subroutines contain each of the JCL decks
# that execute specific mainframe tasks
#
# Each subroutine performs the following functions:
#    1) build JCL file name
#    2) write specific JCL to the file
#    3) execute the JCL
#    4) report status
#
##############################################################



# <<<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< LinkProg >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
sub LinkProg {
    my $status = 0;

		$jcl = 'lnksubmit_' . $mem .'.jcl';
    $jcl = $jcldir . $delimiter . $jcl;
    if(-f $jcl)
    {
    	unlink($jcl);
    }
    
    if ( $attrHash{'LNKDECK'} ne 'N' ){
    	my $linkcntl = $attrHash{'LNKDECK'};
    	my $lopt_parm = 'SIZE=(512K,96K)';
    	if ( $attrHash{'LOPT'} ne 'N' ) {
        $lopt_parm  = $attrHash{'LOPT'};
      }
      bccpbmod::writeFile( $jcl, $compile_1, "
//* *********************************************  LINK-EDIT THE PROGRAM
//* COBOL390 = Y
//LKED     EXEC PGM=IEWL,
//  PARM='$lopt_parm'
//OBJLIB   DD DSN=$name.XOBJLIB,DISP=OLD
//SYSLIN   DD DSN=$name.XLOADLIB($linkcntl),DISP=OLD
"    );

    if ( $env eq $bccpbmod::PROD_env) {			
# PROD
        bccpbmod::writeFile( $jcl, $compile_1, "
//SYSLIB   DD DSN=$syslib1,DISP=SHR
//         DD DSN=CEE.SCEELKED,DISP=SHR
//         DD DSN=PS.UTL.PGMLIB,DISP=SHR
"
         );
    } elsif ( $env eq $bccpbmod::QA_env) {			
 # QA
         bccpbmod::writeFile( $jcl, $compile_1, "
//SYSLIB   DD DSN=$syslib1,DISP=SHR
//         DD DSN=$syslib2,DISP=SHR
//         DD DSN=CEE.SCEELKED,DISP=SHR
//         DD DSN=PS.UTL.PGMLIB,DISP=SHR
"
         );
    } elsif ( $env eq $bccpbmod::DEV_env) {		
 # DEV
          bccpbmod::writeFile( $jcl, $compile_1, "
//SYSLIB   DD DSN=$syslib1,DISP=SHR
//         DD DSN=$syslib2,DISP=SHR
//         DD DSN=$syslib3,DISP=SHR
//*        DD DSN=CEE.SCEELKED,DISP=SHR
//         DD DSN=SYS1.SCEELKED,DISP=SHR
//*        DD DSN=PS.UTL.PGMLIB,DISP=SHR
//*        DD DSN=SYS3.DB2.TEST.DSNLOAD,DISP=SHR
"
         );
     }
     
          # check IMS/DL1
     if ( $attrHash{'DL1'} eq 'Y' ) {
         bccpbmod::writeFile( $jcl, $compile_1, "
//         DD DSN=SYS2.IMSVS.RESLIB.TEST,DISP=SHR
//RESLIB   DD DSN=SYS2.IMSVS.RESLIB.TEST,DISP=SHR
"
         );
     }



        if ( $attrHash{'CICS'} eq 'Y' ) {
            bccpbmod::writeFile(
                $jcl, $compile_1, "
//          DD  DISP=SHR,DSN=CICSTS13.SDFHEXCI
"
            );
        }

        if ( $attrHash{'MQ'} eq 'Y' ) {
            bccpbmod::writeFile(
                $jcl, $compile_1, "
//          DD  DISP=SHR,DSN=MQSERIES.SCSQLOAD
"
            );
        }


        if ( $attrHash{'DB2'} eq 'Y' ) {
            bccpbmod::writeFile(
                $jcl, $compile_1, "
//          DD  DISP=SHR,DSN=DB2.$db2sys.SDSNEXIT
//          DD  DISP=SHR,DSN=DB2.$db2sys.SDSNLOAD
"
            );
        }

        if ( $attrHash{'CICS'} eq 'Y' ) {
            bccpbmod::writeFile(
                $jcl, $compile_1, "
//          DD  DISP=SHR,DSN=PROD.LOAD
"
            );
        }


        if ( $attrHash{'DB2'} eq 'Y' ) {
            bccpbmod::writeFile(
                $jcl, $compile_1, "
//          DD  DISP=SHR,DSN=SYS1.SYSEXEC(INDB2TSO)
"
            );
        }

        if ( $attrHash{'MQ'} eq 'Y' ) {
            bccpbmod::writeFile(
                $jcl, $compile_1, "
//          DD  DISP=SHR,DSN=SYS1.SYSEXEC(INMQBTCH)
"
            );
        }


        bccpbmod::writeFile(
            $jcl, $compile_1, "
//SYSLMOD  DD DSN=$loadlib,
//            DISP=SHR
//SYSUT1   DD UNIT=VIO,SPACE=(CYL,(2,3))
//SYSPRINT DD $sysout
//            $sysout1
"
        );

    }
    if ( $attrHash{'BT'} eq 'Y' ){
    	 bccpbmod::writeFile(
            $jcl, $compile_1, "
//BTTRNLD  EXEC PGM=IKJEFT01
//SYSPRINT DD $sysout
//            $sysout1
//INDD     DD DSN=PSBH.PGMTEST,DISP=SHR
//OUTDD    DD DSN=$name.LDPREP,DISP=SHR
//SYSTSPRT DD $sysout
//            $sysout1
//SYSTSIN  DD *
 XMIT A.A DDNAME(INDD) MEMBER($mem) OUTDDNAME(OUTDD) NOLOG
//XFERLOD  EXEC PGM=IEFBR14
//TEMP     DD DSN=$name.LDPREP,RCCEXT=LOD,DISP=OLD
"
        );
    }
    
    my $cmd =
"\"$bccpbmod::rccbuild\" -h $os390_hostname\@$os390_port -b COPYDO -ft $jcl -n 4 -c LE $rccVerbosity -P " . $mfid . "_". $mem ."_";

    print "Step: linksubmit cmd: $cmd\n " if $debug;

    $status = bccpbmod::rcc_system($compile_1, $cmd);   

    printf( "linksubmit Status was (%d)\n", $status ) if $debug;

    return $status;
}


